/*
    Copyright (C) 2003 by Stephan Linz <linz@li-pro.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Bosten, MA 02111-1307 USA
*/

/* $Id: $ */

/*
   avrctrl/led.h - basic LED bar i/o
 */

#ifndef _AVRCTRL_LED_H_
#define _AVRCTRL_LED_H_ 1

#if defined(HAVE_CONFIG_H) && defined(AVRCTRL_LIB_SRC)
#include <config.h>
#else
#include <avrctrl/config.h>
#endif

#ifndef __ASSEMBLER__

#include <avr/io.h>

/** \defgroup avrctrl_led LED Balken Untersttzung
    \anchor a_avrctrl_led
    \code #include <avrctrl/led.h> \endcode

    Diese Headerdatei deklariert zunchst einen Low-Level Zugang fr den LED
    Balken. Dabei werden immer exakt 8 Bits (LEDs) des LED Ports benutzt. Der
    LED Port ist auf Port B festgelegt.

    Darber hinaus existiert eine zur Laufzeit parametrisierbare LED Grafik
    mit Normierung. Hierbei kann man den Nullpunkt zwischen der untersten und
    der obersten LED frei definieren und wenn ntig nachtrglich verschieben.
    Somit ist die Anzeige von positiven und negativen Werten mglich. Diese
    knnen dann je nach Einstellung als Punkt (eine LED) oder als Balken in
    Bezug zum Nullpunkt dargestellt werden.

    Vor der Benutzung der LED Funktionen, mu dieser Teil der Bibliothek mit
    led_init() initialisiert werden.

    Die Arbeitsweise der LED Grafik wird durch zwei interne Parameter
    bestimmt, welche mit led_graph_control() verndert werden knnen. Alle
    Ausgaben mssen dann mit led_graph_put() erfolgen.

    Mit der Funktion led_graph_control() wird ein Optionsfeld bergeben,
    ssen Bits folgende Bedeutung haben:
    	- Bit 0:
		- rckgesetzt(0): Punktgrafik
		- gesetzt(1): Balkengrafik
	- Bit 3..1:
		- Position des Bezugs- bzw. Nullpunkts
		- Werte zwischen 0 und 7, z.B.:
			- Null(000): unterste LED an Bit 0 des LED Ports
			- Sieben(111): oberste LED an Bit 7 des LED Ports
	- Bit 7..4:
		- bedeutungslos, nicht interpretiert

    \todo Parametrisierung ber mehrere Ports verstreuter Bits. */

/** \name Low-Level Zugriff */

/*@{*/

/** \def led_init()
    \ingroup avrctrl_led
    Initialisiert den LED Port zur Byteausgabe (alle 8 Bits) und setzt
    alle Bits auf den inaktiven Zustand (OFF value).
    
    \return Das Makro led_init() besitzt keinen Rckgabewert. */
#define led_init()					\
{							\
	AVRCTRL_LIB_LED_PORT_DDR = 0xff;		\
	led_cls();					\
}

/** \def led_cls()
    \ingroup avrctrl_led
    Setzt alle Bits auf den inaktiven Zustand (OFF value).
    
    \return Das Makro led_cls() besitzt keinen Rckgabewert. */
#define led_cls()					\
{							\
	AVRCTRL_LIB_LED_PORT = AVRCTRL_LIB_LED_OFF;	\
}

/** \def led_clear()
    \ingroup avrctrl_led
    Macht das Gleiche wie led_cls(). Setzt alle Bits auf den inaktiven
    Zustand (OFF value).
    
    \return Das Makro led_clear() besitzt keinen Rckgabewert. */
#define	led_clear()		led_cls()

/** \def led_put(BITSET)
    \ingroup avrctrl_led
    Setzt alle Bits auf den angegebenen Wert bitset.
    
    \param BITSET ist vom Typ \c unsigned \c char und wird als auszugebendes
    Bitmuster interpretiert.

    \return Das Makro led_put() besitzt keinen Rckgabewert. */
#define led_put(BITSET)					\
{							\
	AVRCTRL_LIB_LED_PORT = (unsigned char)(BITSET);	\
}

/** \def led_get()
    \ingroup avrctrl_led
    Gibt den Wert der aktuell gesetzten LED Ausgabebits zurck.
    
    \return Mit dem Makro led_get() wird das aktuelle Bitmuster vom LED Port
    gelesen und als \c unsigned \c char zurckgegeben. */
#define led_get()					\
	(unsigned char)(AVRCTRL_LIB_LED_PORT_IN)	\

/*@}*/

/** \name Balkengrafik */

/*@{*/

#ifdef __cplusplus
extern "C" {
#endif

/** \fn void led_graph_control(int led_maxlevel, signed char led_options)
    \ingroup avrctrl_led
    \brief Verndert die interne Arbeitsweise der LED Grafik.

    Als Basisinitialisierung der LED Grafik ist die unterste LED als Nullpunkt
    eingestellt (Bit 0) und es gibt keine Normierung. Die Funkltion
    led_graph_put() verhlt sich somit wie led_put().

    Mit der Funktion led_graph_control() kann diese Arbeitsweise verndert
    werden. Hierzu wird der maximal darstellbarer Realwert led_maxlevel fr
    die Normierung und das Optionsfeld led_options bergeben.

    \param led_maxlevel ist der maximal darstellbarer Realwert und wird fr
    die Normierung benutzt. Der Wert mu ungleich Null und grer 8 oder
    kleiner -8 sein. Erfllt der Wert diese Regel nicht, wird die Normierung
    deaktiviert. Der Wert kann auch negativ sein. Dann sollte der Bezugspunkt
    (Nullpunkt) auf einen Wert grer Null gesetzt werden.

    \param led_options definiert die Arbeitsweise der LED Grafik. Siehe
    \ref a_avrctrl_led "Gruppenbeschreibung".

    \return Die Funktion led_graph_control() besitzt keinen Rckgabewert. */
extern void led_graph_control(int led_maxlevel, signed char led_options);

/** \fn void led_graph_put(int bitset_level)
    \ingroup avrctrl_led
    \brief Gibt einen Wert als LED Grafik aus.

    Je nach Parametrisierung durch led_graph_control() wird mit dieser
    Funktion der bergebene Wert zunchst normiert und dann als Grafik
    ausgegeben.
    
    \note Ist keine Normierung eingestellt, so wird der angegebene Wert mit
    led_put() direkt als Bitmuster ausgegeben!

    \param bitset_level ist eine positive oder negative ganze Zahl, die je
    nach Konfiguration der LED Garfik angezeigt wird.

    \return Die Funktion led_graph_put() besitzt keinen Rckgabewert. */
extern void led_graph_put(int bitset_level);

#ifdef __cplusplus
}
#endif

/** \def LED_BAR
    \ingroup avrctrl_led
    Bytewert zum Aktivieren der LED Balkengrafik. */
#define	LED_BAR			(_BV(0))

/** \def LED_POINT
    \ingroup avrctrl_led
    Bytewert zum Aktivieren der LED Punktgrafik. */
#define	LED_POINT		(~(_BV(0)))

/** \def LED_MARGIN()
    \ingroup avrctrl_led
    Bytewert zum Festlegen des Null- bzw. Bezubgspunktes fr die
    LED Grafik. */
#define LED_MARGIN(M)		(((M) & 0x7) << 1)

/*@}*/

#endif /* __ASSEMBLER__ */

#endif /* _AVRCTRL_LED_H_ */
